/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of Number: A Number Detection and 
 * Classification Framework
 * 
 * Number is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar. If not, see <http://www.gnu.org/licenses/>.
 */

#include "AdaptiveThreshold.h"

#include <iostream>
	using std::cout;
	using std::endl;

void adaptiveThreshold( IplImage* dst, IplImage* src, unsigned long* integralImg )
{
	unsigned char* ptrBinary = ( unsigned char* )dst->imageData;

	const int w = src->width;
	const int h = src->height;

	const int S = 128;

	int count = 0;
	int x1, y1, x2, y2;
	int s2 = S >> 1;

	int i, j;
	long sum = 0;
	int srcOffset = 0;
	int dstOffset = 0;  
	const int srcStride = src->height * src->widthStep - src->nChannels;
	const int dstStride = src->height * src->width - 1;
	for ( i = 0; i < src->width; i++ ) {      
		sum = 0;
		for ( j = 0; j < src->height; j++ ) {
			const unsigned char* data = ( const unsigned char* )( &src->imageData[ srcOffset ] );      

			sum += ( 11ul * data[ 0 ] + 16ul * data[ 1 ] + 5ul * data[ 2 ] ) >> 5;

			if ( i == 0 ) {
				integralImg[ dstOffset ] = sum;
			} else {
				integralImg[ dstOffset ] = integralImg[ dstOffset - 1 ] + sum;
			}

			srcOffset += src->widthStep;
			dstOffset += src->width;
		}    
		srcOffset -= srcStride;
		dstOffset -= dstStride;
	}

	// perform thresholding
	srcOffset = 0;
	dstOffset = 0;

	const int stride = src->widthStep - 3 * src->width;
	for ( j = 0; j < src->height; j++ ) {
		for ( i = 0; i < src->width; i++ ) {    
			// set the SxS region
			x1 = i - s2; 
			x2 = i + s2;
			y1 = j - s2; 
			y2 = j + s2;

			// check the border
			if ( x1 < 0 ) x1 = 0;
			if ( x2 >= src->width  ) x2 = src->width  - 1;
			if ( y1 < 0 ) y1 = 0;
			if ( y2 >= src->height ) y2 = src->height - 1;

			count = ( x2 - x1 ) * ( y2 - y1 );

			sum = integralImg[ y2 * src->width + x2 ] -
				integralImg[ y1 * src->width + x2 ] -
				integralImg[ y2 * src->width + x1 ] +
				integralImg[ y1 * src->width + x1 ];

			const unsigned char* data = ( const unsigned char* )( &src->imageData[ srcOffset ] );
			unsigned int color = ( 11ul * data[ 0 ] + 16ul * data[ 1 ] + 5ul * data[ 2 ] ) >> 5;

			if ( color * count * 5 < ( sum << 2 ) ) {
				ptrBinary[ dstOffset ] = 0;
			} else {
				ptrBinary[ dstOffset ] = 255;
			}

			srcOffset += 3;
			dstOffset++;
		}
		srcOffset += stride;
	}
}
