/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of Number: A Number Detection and 
 * Classification Framework
 * 
 * Number is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar. If not, see <http://www.gnu.org/licenses/>.
 */

#include "Blobs.h"
	using std::vector;


void Blobs::extract( vector<Blobs::Blob>& blobs, IplImage* imgBinary, int imgW, int imgH, unsigned char* visited, unsigned int* labelBuffer, PointStack* pointStack )
{
	int label = 0;

	memset( visited, 0, imgW * imgH );  
	
	Point current;  

	unsigned char* ptrBinary = ( unsigned char* )imgBinary->imageData;

	int x, y;
	int offset = 0;
	for ( y = 0; y < imgH; y++ ) {
		for ( x = 0; x < imgW; x++ ) {      
			if ( !visited[ offset ] ) {
				unsigned char ref = ptrBinary[ offset ];

				pointStack->push( Point( x, y, offset ) );               

				int minX = imgW;
				int minY = imgH;
				int maxX = -1;
				int maxY = -1;

				int area = 0;

				bool done = false;

				while ( !done ) {
					done = pointStack->pop( current );          

					if ( !done ) {
						if ( !visited[ current.offset() ] ) 
						{
							visited[ current.offset() ] = 1;

							if ( ptrBinary[ current.offset() ] == ref ) 
							{
								area++;

								labelBuffer[ current.offset() ] = label;

								if ( current.x() < minX ) minX = current.x();
								if ( current.y() < minY ) minY = current.y();
								if ( current.x() > maxX ) maxX = current.x();
								if ( current.y() > maxY ) maxY = current.y();

								if ( ( current.x() < ( imgW - 1 ) ) && ( !visited[ current.offset() + 1 ] ) ) {
									pointStack->push( Point( current.x() + 1, current.y(), current.offset() + 1 ) );
								} 

								if ( ( current.y() < ( imgH - 1 ) ) && ( !visited[ current.offset() + imgW ] ) ) {
									pointStack->push( Point( current.x(), current.y() + 1, current.offset() + imgW ) );
								}

								if ( ( current.x() > 0 ) && ( !visited[ current.offset() - 1 ] ) ) {
									pointStack->push( Point( current.x() - 1, current.y(), current.offset() - 1 ) );
								} 

								if ( ( current.y() > 0 ) && ( !visited[ current.offset() - imgW ] ) ) {
									pointStack->push( Point( current.x(), current.y() - 1, current.offset() - imgW ) );
								}
							} 
						}
					}
				}

				if ( ( minX > 0 ) &&
					( maxX < imgW - 1 ) &&
					( minY > 0 ) &&
					( maxY < imgH - 1 ) &&
					( area > 40 ) &&
					( ( maxY - minY ) > ( maxX - minX ) ) ) 
				{
					Blob blob;
					blob.x = minX;
					blob.y = minY;
					blob.w = maxX - minX + 1;
					blob.h = maxY - minY + 1;
					blob.label = label;

					blobs.push_back( blob );
				} 

				label++;       
			}      
			offset++;
		}
	}
}


void Blobs::classify( vector<Blob>& blobs, Classifier& classifier, unsigned int* labelBuffer, int widthStep )
{
	size_t i;
	for ( i = 0; i < blobs.size(); i++ ) {
		classifier.classify( blobs[ i ].label, 
			labelBuffer, 
			widthStep, 
			blobs[ i ].x, 
			blobs[ i ].y, 
			blobs[ i ].x + blobs[ i ].w - 1, 
			blobs[ i ].y + blobs[ i ].h - 1,
			blobs[ i ].number,
			blobs[ i ].dist );
	}
}