/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of Number: A Number Detection and 
 * Classification Framework
 * 
 * Number is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar. If not, see <http://www.gnu.org/licenses/>.
 */

#include "NumberDemo.h"

#include <math.h>
#include <stdlib.h>

#include <iostream>
	using std::cout;
	using std::cerr;
	using std::endl;
#include <string>
	using std::string;
#include <fstream>
	using std::ifstream;
#include <sstream>
	using std::stringstream;
#include <vector>
	using std::vector;

NumberDemo::NumberDemo() :
	m_imgW( 640 ),
	m_imgH( 480 )
{
	CvSize size = { m_imgW, m_imgH };

	m_imgBinary = cvCreateImage( size, IPL_DEPTH_8U, 1 );

	m_imgLabels = cvCreateImage( size, IPL_DEPTH_8U, 3 );

	m_font = new Font();

	m_labelBuffer = new unsigned int [ m_imgW * m_imgH ];

	m_integralImg = new unsigned long[ m_imgW * m_imgH ];

	m_visited = new unsigned char[ m_imgW * m_imgH ];	

	m_pointStack = new PointStack( 1ul << 20 );

	m_classifier = new Classifier( "data/patterns.dat" );
}

IplImage* NumberDemo::exec( IplImage*img )
{
	m_blobs.clear();

	adaptiveThreshold( m_imgBinary, img, m_integralImg );

	Blobs::extract( m_blobs, m_imgBinary, img->width, img->height, m_visited, m_labelBuffer, m_pointStack );        

	Blobs::classify( m_blobs, *m_classifier, m_labelBuffer, img->width );

	if ( m_mode == 1 ) {
		return img;
	} else if ( m_mode == 2 ) {
		renderBlobs( img, m_blobs, m_font );  
		return img;
	} else if ( m_mode == 3 ) {
		return m_imgBinary;
	} else if ( m_mode == 4 ) {
		mapColors( m_imgLabels, m_labelBuffer, img->width, img->height );
		return m_imgLabels;
	}

	return NULL;
}

void NumberDemo::renderBlobs( IplImage* screen, vector<Blobs::Blob>& blobs, Font* font )
{
	size_t i;
	for ( i = 0; i < blobs.size(); i++ ) {
		if ( blobs[ i ].number == -1 ) {
			renderRectangle( screen, blobs[ i ].x, blobs[ i ].y, blobs[ i ].x + blobs[ i ].w - 1, blobs[ i ].y + blobs[ i ].h - 1, 255,   0, 0 );
		} else {
			renderRectangle( screen, blobs[ i ].x, blobs[ i ].y, blobs[ i ].x + blobs[ i ].w - 1, blobs[ i ].y + blobs[ i ].h - 1,   0, 255, 0 );
		}   

		stringstream numberStr;
		numberStr << blobs[ i ].number;    
		font->ShowText( numberStr.str(), 1, blobs[ i ].x, blobs[ i ].y, screen );

		stringstream distStr;
		distStr << blobs[ i ].dist;    
		font->ShowText( distStr.str(), 1, blobs[ i ].x, blobs[ i ].y + 20, screen );    
	}   
}

void NumberDemo::renderRectangle( IplImage* screen, int x1, int y1, int x2, int y2, int r, int g, int b )
{
	unsigned char* pixels = ( unsigned char* )screen->imageData;

	int x;
	for ( x = x1; x <= x2; x++ ) {
		int offsetA = y1 * screen->widthStep + x * screen->nChannels;
		int offsetB = y2 * screen->widthStep + x * screen->nChannels;

		pixels[ offsetA     ] = b;
		pixels[ offsetA + 1 ] = g;
		pixels[ offsetA + 2 ] = r;

		pixels[ offsetB     ] = b;
		pixels[ offsetB + 1 ] = g;
		pixels[ offsetB + 2 ] = r;
	}

	int y;
	for ( y = y1; y <= y2; y++ ) {
		int offsetA = y * screen->widthStep + x1 * screen->nChannels;
		int offsetB = y * screen->widthStep + x2 * screen->nChannels;

		pixels[ offsetA     ] = b;
		pixels[ offsetA + 1 ] = g;
		pixels[ offsetA + 2 ] = r;

		pixels[ offsetB     ] = b;
		pixels[ offsetB + 1 ] = g;
		pixels[ offsetB + 2 ] = r;
	}
}

void NumberDemo::mapColors( IplImage* dst, unsigned int* src, int w, int h )
{
	static const unsigned char colors[ 12 ][ 3 ] =
	{ { 255,   0,   0 },
	  {   0, 255,   0 },
	  {   0,   0, 255 },
	  { 255, 255,   0 },
	  {   0, 255, 255 },
	  { 255,   0, 255 },
	  { 128,   0,   0 },
	  {   0, 128,   0 },
	  {   0,   0, 128 },
	  { 128, 128,   0 },
	  {   0, 128, 128 },
	  { 128,   0, 128 } 
	};

	unsigned char* ptr = ( unsigned char* )dst->imageData;

	int x, y;
	int srcOffset = 0;
	int dstOffset = 0;  
	for ( y = 0; y < h; y++ ) {
		for ( x = 0; x < w; x++ ) {
			ptr[ dstOffset     ] = colors[ src[ srcOffset ] % 12 ][ 0 ];
			ptr[ dstOffset + 1 ] = colors[ src[ srcOffset ] % 12 ][ 1 ];
			ptr[ dstOffset + 2 ] = colors[ src[ srcOffset ] % 12 ][ 2 ];

			srcOffset ++;

			dstOffset += 3;
		}
	}
} 