/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of Number: A Number Detection and 
 * Classification Framework
 * 
 * Number is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __POINT_STACK_H__
#define __POINT_STACK_H__

#include <iostream>

class Point
{
  public:
    
    inline Point();
    
    inline Point( int x, int y, int offset );
    
    inline int x() const;
    
    inline int y() const;
    
    inline int offset() const;
    
  protected:
    
    int m_x;
    int m_y;
    int m_offset;
};


inline Point::Point() :
  m_x( 0 ),
  m_y( 0 ),
  m_offset( 0 )
{
}


inline Point::Point( int x, int y, int offset ) :
  m_x( x ),
  m_y( y ),
  m_offset( offset )
{
}


inline int Point::x() const
{
  return m_x;
}


inline int Point::y() const
{
  return m_y;
}


inline int Point::offset() const
{
  return m_offset;
}


class PointStack
{
  public:
    
    inline PointStack( int size );
    
    inline ~PointStack();
    
    inline void push( const Point& point );
    
    inline bool pop( Point& point );
    
    inline void clear();
    
  protected:
    
    int m_size;
    
    int m_pointer;
    
    Point* m_data;
};


inline PointStack::PointStack( int size ) :
  m_size( size ),
  m_pointer( 0 )
{
  m_data = new Point[ size ];
}


inline PointStack::~PointStack()
{
  delete [] m_data;  
  m_data = NULL;
}

    
inline void PointStack::push( const Point& point )
{
  m_data[ m_pointer ] = point;
  
  m_pointer++;
  
  if ( m_pointer == m_size ) {    
    Point* data = new Point[ m_size << 1 ];
    memcpy( data, m_data, m_size * sizeof( Point ) );
    delete [] m_data;
    m_data = data;    
    m_size <<= 1;
    std::cout << "pointStack size is now " << m_size << std::endl;
  }
}
 
 
inline bool PointStack::pop( Point& point )
{
  if ( m_pointer > 0 ) {
    m_pointer--;
    point = m_data[ m_pointer ];
    return false;
  } else {
    return true;
  }
}


inline void PointStack::clear()
{
  m_pointer = 0;
}


#endif