/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of Number: A Number Detection and 
 * Classification Framework
 * 
 * Number is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar. If not, see <http://www.gnu.org/licenses/>.
 */

#include <opencv2/opencv.hpp>
#include <opencv2/highgui/highgui.hpp>

#include <iostream>
	using std::cout;
	using std::cerr;
	using std::endl;

#include <stdlib.h>

#include <float.h>

#include <SDL.h>

#include "NumberDemo.h"


void ipl2sdl( IplImage* src, SDL_Surface* dst )
{
	if ( src->width != dst->w ) {
		cerr << "src and dst images provide varying widths: " << src->width << " vs. " << dst->w << endl;
		return;
	}

	unsigned char* dstData = ( unsigned char* )dst->pixels;

	SDL_LockSurface( dst );

	int srcOffset = 0;
	int dstOffset = 0;

	if ( src->nChannels == dst->format->BytesPerPixel ) {
		int y;
		for ( y = 0; y < src->height; y++ ) {
			memcpy( &dstData[ dstOffset ], &src->imageData[ srcOffset ], src->width * src->nChannels );

			srcOffset += src->widthStep;
			dstOffset += dst->pitch;
		}		
	} else if ( ( src->nChannels == 1 ) && ( dst->format->BytesPerPixel == 3 ) ) {
		int x, y;
		for ( y = 0; y < src->height; y++ ) {
			int auxSrc = srcOffset;
			int auxDst = dstOffset;
			for ( x = 0; x < src->width; x++ ) {
				dstData[ auxDst     ] = src->imageData[ auxSrc ];
				dstData[ auxDst + 1 ] = src->imageData[ auxSrc ];
				dstData[ auxDst + 2 ] = src->imageData[ auxSrc ];
				
				auxSrc++;

				auxDst += 3;				
			}
			srcOffset += src->widthStep;
			dstOffset += dst->pitch;
		}
	} else {
		cerr << "src and dst images provide varying depths: " << src->nChannels << " vs. " << dst->format->BytesPerPixel << endl;
	}

	SDL_UnlockSurface( dst );
}


bool processEvents( Program* demo )
{
	SDL_Event event;

	bool done = false;

	while( SDL_PollEvent( &event ) ) {
		if ( event.type == SDL_KEYDOWN ) {
			if ( event.key.keysym.sym == SDLK_ESCAPE ) {
				done = true;
			} else if ( event.key.keysym.sym == SDLK_1 ) {
				demo->setMode( 1 );
			} else if ( event.key.keysym.sym == SDLK_2 ) {
				demo->setMode( 2 );
			} else if ( event.key.keysym.sym == SDLK_3 ) {
				demo->setMode( 3 );
			} else if ( event.key.keysym.sym == SDLK_4 ) {
				demo->setMode( 4 );
			} 
		}    
	}

	return done;
}


int main(int argc, char *argv[])
{
	SDL_Surface* screen;

	if ( SDL_Init( SDL_INIT_VIDEO ) == -1 ) {
		printf( "Can't init SDL:  %s\n", SDL_GetError() );
		exit( 1 );
	}

	screen = SDL_SetVideoMode( 640, 480, 24, SDL_HWSURFACE );
	if (screen == NULL) {
		printf( "Can't set video mode: %s\n", SDL_GetError() );
		exit( 1 );
	}

	IplImage* img = NULL;

	IplImage* imgResized = NULL; 

	Program* liveDemo = new NumberDemo();

	CvCapture* cv_cap = cvCaptureFromCAM( 0 );

	double capW = cvGetCaptureProperty( cv_cap, CV_CAP_PROP_FRAME_WIDTH );
	double capH = cvGetCaptureProperty( cv_cap, CV_CAP_PROP_FRAME_HEIGHT );

	bool done = false;
	while ( !done ) {
		img = cvQueryFrame( cv_cap );

		if ( !imgResized ) {
			CvSize imgSize = { 640, 480 };			
			imgResized = cvCreateImage( imgSize, IPL_DEPTH_8U, 3 );
		}

		cvResize( img, imgResized );

		IplImage* imgDemo = liveDemo->exec( imgResized );	

		ipl2sdl( imgDemo, screen );

		SDL_Flip( screen );

		done = processEvents( liveDemo );
	}

	/* clean up */
	cvReleaseCapture( &cv_cap );

	return 0;
}